/*
 * Handles linear constraint rules
 */

package checkpolicy.prolog;

import checkpolicy.util.*;

import java.util.*;

public class LinearConstraintRule extends Rule
{

    // Attributes
    private HashMap<String, PairTuple<String,String>> bounds =
            new HashMap<String, PairTuple<String,String>>();
    private TripleTuple[] constraint = new TripleTuple[0];

    /**
     * Generates a linear constraint solver (note that it is assumed that
     * ":- use_module(library('clp/bounds'))." appears elsewhere in the
     * application).
     * @param functor The name of the predicate.
     * @param args A list of string arguments.
     * @param bounds Mappings of variables from names to upper and lower bounds
     * (first = lower bound, second = upper bound).
     * @param constraint Constraints as PairTuples (first = left-hand side,
     * second = right-hand side). Treated as a disjunctive set of equalities.
     */
    public LinearConstraintRule(
            String functor,
            String[] args,
            HashMap<String, PairTuple<String,String>> bounds,
            TripleTuple[] constraint)
    {
        setFunctor(functor);
        setArguments(args);
        this.bounds = bounds;
        this.constraint = constraint;
    }
    
    /**
     * Obtains the full call information (name + args) to be sent to the
     * Query mechanism.
     * @return The string to be submitted as a query.
     */
    public String getCallString()
    {
        // Create a comma-delimited list of all args
        StringBuilder commaListSB = new StringBuilder();
        String[] args = getArguments();
        for (int i = 0; i < args.length; i++)
        {
            commaListSB.append(args[i]);
            if (i < args.length-1)
                commaListSB.append(",");
        }
        String commaList = commaListSB.toString();
        
        return (getFunctor() + "(" + commaList + ").");
    }

    /**
     * Constructs the Prolog code for the linear constraint solver rule.
     * @return The code for the solver, to be inserted in a Prolog file.
     */
    @Override
    public String toString()
    {
        StringBuilder s = new StringBuilder();
        
        // Create a comma-delimited list of all args
        StringBuilder commaListSB = new StringBuilder();
        String[] args = getArguments();
        for (int i = 0; i < args.length; i++)
        {
            commaListSB.append(args[i]);
            if (i < args.length-1)
                commaListSB.append(",");
        }
        String commaList = commaListSB.toString();
        
        // Start the rule
        s.append(getFunctor() + "(" + commaList + ") :-\n\n");
        s.append("  Nums = [" + commaList + "],\n\n");
        
        // Construct the lower and upper boundaries
        Iterator<String> iter = bounds.keySet().iterator();
        while (iter.hasNext())
        {
            String key = iter.next();
            s.append("  " + key + " #>= " + bounds.get(key).first + ",\n" +
                     "  " + key + " #=< " + bounds.get(key).second + ",\n");
        }
        s.append("\n");
        
        // Now the individual constraints
        s.append("  (\n");
        boolean addSemicolon = false;
        for (int i = 0; i < constraint.length; i++)
        {
            if (addSemicolon)
                s.append(",\n");
            s.append("    " + constraint[i].first + " " + constraint[i].second + " " + constraint[i].third);
            addSemicolon = true;
        }
        s.append("\n  ),\n\n");
        
        // Finish up
        s.append("  label(Nums).\n\n");
        
        return s.toString();
    }

}
